<?php

/**
 * Inspired by the "Routing" module of Rails framework, this classe enable you
 * to define URL pattern and connect each of them to an array of value. Then,
 * with a given URL the classe can extract all its variables.
 *
 * With some additional work, you can easily do pseudo URL rewriting without
 * using apache mod_rewrite... 
 *
 * For you can have those URLs for a little blog script:
 *
 *  /articles/list          # list all articles
 *  /articles/view/3        # view article which id = 3
 *  /articles/edit/3        # edit article which id = 3
 *  /articles/3/add/comment # add a comment for article which id = 3
 *
 * That result in this mapping:
 *
 *  $routes = new Routes();
 *  $routes->connect('/article/list', array('action' => 'list'));
 *  $routes->connect('/article/view/:id', array('action' => 'view'));
 *  $routes->connect('/article/edit/:id', array('action' => 'edit'));
 *  $routes->connect('/article/:id/add/comment', array('action' => 'comment'));
 *
 * Then you can extract information from URLS:
 *
 *  $routes->extract('/article/list') 
 *  # return: array('action' => 'list')
 *
 *  $routes->extract('/article/view/2') 
 *  # return: array('action' => 'view', 'id' => '2')
 *
 *  $routes->extract('/article/2/add/comment')
 *  # return: array('action' => 'comment', 'id' => '2')
 *
 * You can use regex:
 *
 *  $routes = new Routes();
 *  $routes->connect('/article/:year', 
 *    array('action' => 'list'), 
 *    array('year' => '/\d{4}/'));
 *  $routes->connect('/article/:year/:month', 
 *    array('action' => 'list'), 
 *    array('year' => '/\d{4}/', 'month' => '/\d{1,2}/'));
 *
 * facultative segment:
 *
 *  $routes = new Routes();
 *  $routes->connect('/article/:page?', array('page' => 0));
 * 
 * If you need to define more complexe patterns you can describe the complete URL 
 * with a single regular expression :
 * 
 *  $routes = new Routes();
 *  $routes->connect('/^(?P<category>.*)/(?P<page>\d+)$/',
 *    array('action' => 'category'));
 *  print_r($routes->extract('/cat/subcat/3'));
 *
 * Warning: When more than one pattern match a URL the first connected pattern
 * is used to extract values!
 */
class Routes {

    /**
     * All avaible routes are stocked in a tree in order to maximize performace.
     */
    var $tree = array();

    /**
     * Constantes used for the tree building.
     */
    var $PATTERN = '__pattern__';
    var $NAME    = '__name__';
    var $VARS    = '__vars__';

    function Routes($cache=null) {
        if ($cache != null) {
            $this->import($cache);
        }
    }
    
    /**
     * Define a new route.
     */
    function connect($url, $vars=array(), $regexps=array()) {
        // remove begining "/" if present
        if ($url && $url[0] == '/') {
            $url = substr($url, 1);
        }
        
        // remove faculatitive ending "/"
        if ($url && $url[strlen($url)-1] == '/') {
            $url = substr($url, 0, -1);
        }
        
        // if it's an url regex
        if ($url && $url[0] == '^') {
        	$this->tree[] = array(
        		$this->PATTERN => $url,
        		$this->VARS => $vars,
        	);
        	return;
        }

        // add the route in the tree
        $this->addRoute($this->tree, explode('/', $url), $vars, $regexps);
    }

    /**
     * Extract all variables of the given URL.
     */
    function extract($url) {
        // remove faculatitive ending "/"
        if ($url && $url[strlen($url)-1] == '/') {
            $url = substr($url, 0, -1);
        }
        
        // remove begining "/" if present
        if ($url && $url[0] == '/') {
            $url = substr($url, 1);
        }

        return $this->recognize($this->tree, explode('/', $url));
    }

    /**
     * Return routes tree.
     */
    function export() { return $this->tree; }

    /**
     * Import a routes tree.
     */
    function import($data) { $this->tree = $data; }
    
    /**
     * Used for debugging
     */
    function __toString() { return print_r($this->tree, true); }

    /**
     * Add a route in the tree.
     */
    function addRoute(&$tree, $segments, $vars, $regexps) {                
        $segment = array_shift($segments);
        $array = array();

        // if it's a facultative segment
        if ($segment && $segment[strlen($segment)-1] == '?') {
            $segment = substr($segment, 0, -1); 
            
            // add route for the URL with this segment
            array_unshift($segments, $segment);
            $this->addRoute($tree, $segments, $vars, $regexps);
            array_shift($segments);
            
            // add a route for the URL without this segment
            if (count($segments) > 0) {
                $this->addRoute($tree, $segments, $vars, $regexps);
            } else {
                $tree[$this->VARS] = $vars;
            }

            return;
        }
        
        // if it's a variable segment
        if ($segment && $segment[0] == ':') {
            $segment = substr($segment, 1);
            $tmp = $segment;
            if (isset($regexps[$segment])) {
                $segment = $regexps[$segment];
            } else {
                $segment = '/(.+)/';
            }
            $array[$this->NAME] = $tmp; 
        }
        
        $array[$this->PATTERN] = $segment;

        if (count($segments) > 0) {
            foreach ($tree as $key => $value) {
                if (is_array($value) && isset($value[$this->PATTERN]) && $value[$this->PATTERN] == $segment) {
                    if ((isset($value[$this->NAME]) && isset($array[$this->NAME]) && $value[$this->NAME] == $array[$this->NAME]) ||
                            (!isset($value[$this->NAME]) && !isset($array[$this->NAME]))) {
                        return $this->addRoute($tree[$key], $segments, $vars, $regexps);
                    }
                }
            }
            $tree[] =& $array;
            $this->addRoute($array, $segments, $vars, $regexps);
        } else {
            $array[$this->VARS] = $vars;
            $tree[] = $array;
        }
    }

    /**
     * Find a path in the tree which match the URL, return all vars associated with the URL.
     */
    function recognize(&$tree, $segments) {
        if (count($segments) == 0) {
            if (!isset($tree[$this->VARS])) {
                return false;
            }
            return $tree[$this->VARS];
        } else {
            $segment = array_shift($segments);

            foreach ($tree as $k => $elt) {
                if (is_array($elt) && $k !== $this->VARS) {
                    if ($elt[$this->PATTERN] && $elt[$this->PATTERN][0] == '/') {
                        // it's a segment regex
                        $matches = null;
                        if (preg_match($elt[$this->PATTERN], $segment, $matches) == 1) {
                        	// segment is recognized
                            $r = $this->recognize($elt, $segments);
                            if ($r !== false) return array_merge($r, array($elt[$this->NAME] => $matches[0]));
                        }
                    } elseif ($elt[$this->PATTERN] && $elt[$this->PATTERN][0] == '^') {
                    	// it's an url regex
                        $matches = null;
                        $url = implode('/', $segments);
                        if ($segment) $url = $segment.($url ? '/'.$url : '');
                        if (preg_match('@'.$elt[$this->PATTERN].'@', $url, $matches)) {
                        	// url is recognized
                        	$result = array();
                        	// $matches filter
                        	foreach ($matches as $key => $value) {
                        		if (is_string($key)) $result[$key] = $value;
                        	}
                        	return array_merge($elt[$this->VARS], $result);
                        }
                    } else {
                        // it's a string
                        if ($elt[$this->PATTERN] == $segment) {
                            $r = $this->recognize($elt, $segments);
                            if ($r !== false) return $r;
                        }
                    }
                }
            }
            return false;
        }
    }
}
?>
