<?php
require_once dirname(__FILE__).'/../lib/routes.php';

$sp = dirname(__FILE__).'/../../../simpletest/';
require_once $sp.'unit_tester.php';
require_once $sp.'reporter.php';

class RoutesTest extends UnitTestCase {

    function testStaticUrls() {
        $routes = new Routes();
        $routes->connect('/images/view', array('a' => 'a'));
        $routes->connect('/images/list', array('a' => 'a', 'b' => 'b'));
        
        $this->assertEqual($routes->extract('/images/view'), array('a' => 'a'));
        $this->assertEqual($routes->extract('/images/list'), array('a' => 'a', 'b' => 'b'));

        $routes = new Routes();
        $routes->connect('images/view', array('a' => 'a'));
        $routes->connect('images/list', array('a' => 'a', 'b' => 'b'));
        
        $this->assertEqual($routes->extract('images/view'), array('a' => 'a'));
        $this->assertEqual($routes->extract('images/list'), array('a' => 'a', 'b' => 'b'));
    }
    
    function testSpecialUrls() {
        $routes = new Routes();
        $routes->connect('', array('a' => 'a'));
        $routes->connect('/test', array('b' => 'b'));
        
        $this->assertEqual($routes->extract(''), array('a' => 'a'));
        $this->assertEqual($routes->extract('/test/'), array('b' => 'b'));
        $this->assertEqual($routes->extract('test/'), array('b' => 'b'));
        $this->assertEqual($routes->extract('test'), array('b' => 'b'));
    }

    function testStaticAndDynamicUrls() {
        $routes = new Routes();
        $routes->connect('/images/view', array('a' => 'a'));
        $routes->connect('/images/list', array('a' => 'a', 'b' => 'b'));
        $routes->connect('/images/:var', array('c' => 'c'));
        
        $this->assertEqual($routes->extract('/images/view'), array('a' => 'a'));
        $this->assertEqual($routes->extract('/images/list'), array('a' => 'a', 'b' => 'b'));
        $this->assertEqual($routes->extract('/images/2222'), array('c' => 'c', 'var' => '2222'));
    }

    function testDynamicUrls() {
        $routes = new Routes();
        $routes->connect('/images/:var', array('b' => 'b'));
        $routes->connect('/:var/images', array('a' => 'a'));
        
        $this->assertEqual($routes->extract('/view/images'), array('a' => 'a', 'var' => 'view'));
        $this->assertEqual($routes->extract('/images/view'), array('b' => 'b', 'var' => 'view'));
        
        $routes = new Routes();
        $routes->connect('/images/:list/33');
        $routes->connect('/images/:edit');

        $this->assertEqual($routes->extract('/images/test/33'), array('list' => 'test'));
        $this->assertEqual($routes->extract('/images/test'), array('edit' => 'test'));
        
        $routes = new Routes();
        $routes->connect('/images/:list/10/aaaaa');
        $routes->connect('/images/:list/10/:test');

        $this->assertEqual($routes->extract('/images/a/10/aa'), array('list' => 'a', 'test' => 'aa'));
        $this->assertEqual($routes->extract('/images/b/10/aaaaa'), array('list' => 'b'));
    }

    function testDynamicWithRegexUrls() {
        $routes = new Routes();
        $routes->connect('/2005/:month/:day', array(), array('month' => '/\d{1,2}/', 'day' => '/\d{1,2}/'));
        $routes->connect('/:year', array(), array('year' => '/\d{4}/'));
        $routes->connect('/:year/:month', array(), array('year' => '/\d{4}/', 'month' => '/\d{1,2}/'));
        $routes->connect('/:year/:month/:day', array(), array('year' => '/\d{4}/', 'month' => '/\d{1,2}/', 'day' => '/\d{1,2}/'));
            
        $this->assertEqual($routes->extract('/2005'), array('year' => '2005'));
        $this->assertEqual($routes->extract('/2005/12'), array('year' => '2005', 'month' => '12'));
        $this->assertEqual($routes->extract('/2005/12/25'), array('month' => '12', 'day' => '25'));
        $this->assertEqual($routes->extract('/2006/12/25'), array('year' => '2006', 'month' => '12', 'day' => '25'));
    }

    function testInvalidRoutes() {
        $routes = new Routes();
        $routes->connect('/images/list');
        
        $this->assertFalse($routes->extract('/blabla'), false);
        $this->assertEqual($routes->extract('/images/list'), array());
    }

    function testImportExportRoutes() {
        $routes = new Routes();
        $routes->connect('/images/view', array('a' => 'a'));
        $routes = new Routes($routes->export());
        $this->assertEqual($routes->extract('/images/view'), array('a' => 'a'));
    }

    function testFacultativeSegment() {
        $routes = new Routes();
        $routes->connect('/article/:page?', array('page' => 0));
        $this->assertEqual($routes->extract('/article'), array('page' => 0));
        $this->assertEqual($routes->extract('/article/1'), array('page' => 1));
        
        $routes = new Routes();
        $routes->connect('/:page?/numero', array('page' => 0));
        $this->assertEqual($routes->extract('/numero'), array('page' => 0));
        $this->assertEqual($routes->extract('/10/numero'), array('page' => 10));
        
        $routes = new Routes();
        $routes->connect('/:p1?/:p2?/:p3?', array('p1' => 1, 'p2' => 2, 'p3' => 3));
        $this->assertEqual($routes->extract('/10'), array('p1' => 10, 'p2' => 2, 'p3' => 3));
        $this->assertEqual($routes->extract('/10/20'), array('p1' => 10, 'p2' => 20, 'p3' => 3));
        $this->assertEqual($routes->extract('/10/20/30'), array('p1' => 10, 'p2' => 20, 'p3' => 30));
    }

    function testRegexUrl() {
        $routes = new Routes();
        $routes->connect('/^(?P<category>.*)$/');
        $this->assertEqual($routes->extract(''), array('category' => ''));
        $this->assertEqual($routes->extract('/c1/c2/c3/c4'), array('category' => 'c1/c2/c3/c4'));
         
        $routes = new Routes();
        $routes->connect('/^(?P<category>.*)/(?P<page>\d+)$/', array('test' => 1));
        $this->assertEqual($routes->extract('/c1/c2/10'), array('category' => 'c1/c2', 'page' => 10, 'test' => 1));
        $this->assertFalse($routes->extract('/c1/c2/c3'));
         
        $routes = new Routes();
        $routes->connect('/^(?P<category>.+)/(?P<subcat>.+)$/');
        $this->assertEqual($routes->extract('/c1/c2/sc1'), array('category' => 'c1/c2', 'subcat' => 'sc1'));
        $this->assertfalse($routes->extract('/c1'));
         
        $routes = new Routes();
        $routes->connect('/^nature/(?P<category>.+)/(?P<page>\d+)$/');
        $this->assertEqual($routes->extract('/nature/c1/c2/c3/1'), array('category' => 'c1/c2/c3', 'page' => '1'));
         
        $routes = new Routes();
        $routes->connect('/^var(?P<page>\d+)$/');
        $this->assertEqual($routes->extract('/var10'), array('page' => 10));
         
        $routes = new Routes();
        $routes->connect('/^p-(?P<page>\d+)_i-(?P<image>\d+)$/');
        $this->assertEqual($routes->extract('/p-5_i-6'), array('page' => 5, 'image' => 6));
    }
}

if (isset($argv) && basename($argv[0]) == basename(__FILE__)) {
    $test = new RoutesTest('RoutesTest');
    if (TextReporter::inCli()) {
        exit ($test->run(new TextReporter()) ? 0 : 1);
    }
    $test->run(new HtmlReporter());
}
?>
